searchData={"items":[{"type":"module","title":"Jido","doc":"Jido is a flexible framework for building distributed AI Agents and Workflows in Elixir.  It enables intelligent automation in Elixir, with a focus on Actions, Workflows, Bots, Agents, Sensors, and Signals for creating dynamic and adaptive systems.\n\nThis module provides the main interface for interacting with Jido components, including:\n- Listing and retrieving Actions, Sensors, and Domains\n- Filtering and paginating results\n- Generating unique slugs for components","ref":"Jido.html"},{"type":"module","title":"Examples - Jido","doc":"iex> Jido.list_actions()\n    [%{module: MyApp.SomeAction, name: \"some_action\", description: \"Does something\", slug: \"abc123de\"}]\n\n    iex> Jido.get_action_by_slug(\"abc123de\")\n    %{module: MyApp.SomeAction, name: \"some_action\", description: \"Does something\", slug: \"abc123de\"}","ref":"Jido.html#module-examples"},{"type":"function","title":"Jido.get_action_by_slug/1","doc":"Retrieves an Action by its slug.","ref":"Jido.html#get_action_by_slug/1"},{"type":"function","title":"Parameters - Jido.get_action_by_slug/1","doc":"- `slug`: A string representing the unique identifier of the Action.","ref":"Jido.html#get_action_by_slug/1-parameters"},{"type":"function","title":"Returns - Jido.get_action_by_slug/1","doc":"The Action metadata if found, otherwise `nil`.","ref":"Jido.html#get_action_by_slug/1-returns"},{"type":"function","title":"Examples - Jido.get_action_by_slug/1","doc":"iex> Jido.get_action_by_slug(\"abc123de\")\n    %{module: MyApp.SomeAction, name: \"some_action\", description: \"Does something\", slug: \"abc123de\"}\n\n    iex> Jido.get_action_by_slug(\"nonexistent\")\n    nil","ref":"Jido.html#get_action_by_slug/1-examples"},{"type":"function","title":"Jido.get_agent_by_slug/1","doc":"Retrieves an Agent by its slug.","ref":"Jido.html#get_agent_by_slug/1"},{"type":"function","title":"Parameters - Jido.get_agent_by_slug/1","doc":"- `slug`: A string representing the unique identifier of the Agent.","ref":"Jido.html#get_agent_by_slug/1-parameters"},{"type":"function","title":"Returns - Jido.get_agent_by_slug/1","doc":"The Agent metadata if found, otherwise `nil`.","ref":"Jido.html#get_agent_by_slug/1-returns"},{"type":"function","title":"Examples - Jido.get_agent_by_slug/1","doc":"iex> Jido.get_agent_by_slug(\"ghi789jk\")\n    %{module: MyApp.SomeAgent, name: \"some_agent\", description: \"Represents an agent\", slug: \"ghi789jk\"}\n\n    iex> Jido.get_agent_by_slug(\"nonexistent\")\n    nil","ref":"Jido.html#get_agent_by_slug/1-examples"},{"type":"function","title":"Jido.get_sensor_by_slug/1","doc":"Retrieves a Sensor by its slug.","ref":"Jido.html#get_sensor_by_slug/1"},{"type":"function","title":"Parameters - Jido.get_sensor_by_slug/1","doc":"- `slug`: A string representing the unique identifier of the Sensor.","ref":"Jido.html#get_sensor_by_slug/1-parameters"},{"type":"function","title":"Returns - Jido.get_sensor_by_slug/1","doc":"The Sensor metadata if found, otherwise `nil`.","ref":"Jido.html#get_sensor_by_slug/1-returns"},{"type":"function","title":"Examples - Jido.get_sensor_by_slug/1","doc":"iex> Jido.get_sensor_by_slug(\"def456gh\")\n    %{module: MyApp.SomeSensor, name: \"some_sensor\", description: \"Monitors something\", slug: \"def456gh\"}\n\n    iex> Jido.get_sensor_by_slug(\"nonexistent\")\n    nil","ref":"Jido.html#get_sensor_by_slug/1-examples"},{"type":"function","title":"Jido.list_actions/1","doc":"Lists all Actions with optional filtering and pagination.","ref":"Jido.html#list_actions/1"},{"type":"function","title":"Parameters - Jido.list_actions/1","doc":"- `opts`: A keyword list of options for filtering and pagination. Available options:\n  - `:limit`: Maximum number of results to return.\n  - `:offset`: Number of results to skip before starting to return.\n  - `:name`: Filter Actions by name (partial match).\n  - `:description`: Filter Actions by description (partial match).\n  - `:category`: Filter Actions by category (exact match).\n  - `:tag`: Filter Actions by tag (must have the exact tag).","ref":"Jido.html#list_actions/1-parameters"},{"type":"function","title":"Returns - Jido.list_actions/1","doc":"A list of Action metadata.","ref":"Jido.html#list_actions/1-returns"},{"type":"function","title":"Examples - Jido.list_actions/1","doc":"iex> Jido.list_actions(limit: 10, offset: 5, category: :utility)\n    [%{module: MyApp.SomeAction, name: \"some_action\", description: \"Does something\", slug: \"abc123de\", category: :utility}]","ref":"Jido.html#list_actions/1-examples"},{"type":"function","title":"Jido.list_agents/1","doc":"Lists all Agents with optional filtering and pagination.","ref":"Jido.html#list_agents/1"},{"type":"function","title":"Parameters - Jido.list_agents/1","doc":"- `opts`: A keyword list of options for filtering and pagination. Available options:\n  - `:limit`: Maximum number of results to return.\n  - `:offset`: Number of results to skip before starting to return.\n  - `:name`: Filter Agents by name (partial match).\n  - `:description`: Filter Agents by description (partial match).\n  - `:category`: Filter Agents by category (exact match).\n  - `:tag`: Filter Agents by tag (must have the exact tag).","ref":"Jido.html#list_agents/1-parameters"},{"type":"function","title":"Returns - Jido.list_agents/1","doc":"A list of Agent metadata.","ref":"Jido.html#list_agents/1-returns"},{"type":"function","title":"Examples - Jido.list_agents/1","doc":"iex> Jido.list_agents(limit: 10, offset: 5, category: :business)\n    [%{module: MyApp.SomeAgent, name: \"some_agent\", description: \"Represents an agent\", slug: \"ghi789jk\", category: :business}]","ref":"Jido.html#list_agents/1-examples"},{"type":"function","title":"Jido.list_sensors/1","doc":"Lists all Sensors with optional filtering and pagination.","ref":"Jido.html#list_sensors/1"},{"type":"function","title":"Parameters - Jido.list_sensors/1","doc":"- `opts`: A keyword list of options for filtering and pagination. Available options:\n  - `:limit`: Maximum number of results to return.\n  - `:offset`: Number of results to skip before starting to return.\n  - `:name`: Filter Sensors by name (partial match).\n  - `:description`: Filter Sensors by description (partial match).\n  - `:category`: Filter Sensors by category (exact match).\n  - `:tag`: Filter Sensors by tag (must have the exact tag).","ref":"Jido.html#list_sensors/1-parameters"},{"type":"function","title":"Returns - Jido.list_sensors/1","doc":"A list of Sensor metadata.","ref":"Jido.html#list_sensors/1-returns"},{"type":"function","title":"Examples - Jido.list_sensors/1","doc":"iex> Jido.list_sensors(limit: 10, offset: 5, category: :monitoring)\n    [%{module: MyApp.SomeSensor, name: \"some_sensor\", description: \"Monitors something\", slug: \"def456gh\", category: :monitoring}]","ref":"Jido.html#list_sensors/1-examples"},{"type":"type","title":"Jido.component_metadata/0","doc":"","ref":"Jido.html#t:component_metadata/0"},{"type":"behaviour","title":"Jido.Action","doc":"Defines a discrete, composable unit of functionality within the Jido system.\n\nEach Action represents a delayed computation that can be composed with others\nto build complex workflows and workflows. Actions are defined at compile-time\nand provide a consistent interface for validating inputs, executing workflows,\nand handling results.","ref":"Jido.Action.html"},{"type":"behaviour","title":"Features - Jido.Action","doc":"- Compile-time configuration validation\n- Runtime input parameter validation\n- Consistent error handling and formatting\n- Extensible lifecycle hooks\n- JSON serialization support","ref":"Jido.Action.html#module-features"},{"type":"behaviour","title":"Usage - Jido.Action","doc":"To define a new Action, use the `Jido.Action` behavior in your module:\n\n    defmodule MyAction do\n      use Jido.Action,\n        name: \"my_action\",\n        description: \"Performs a specific workflow\",\n        category: \"processing\",\n        tags: [\"example\", \"demo\"],\n        vsn: \"1.0.0\",\n        schema: [\n          input: [type: :string, required: true]\n        ]\n\n      @impl true\n      def run(params, _context) do\n        # Your action logic here\n        {:ok, %{result: String.upcase(params.input)}}\n      end\n    end","ref":"Jido.Action.html#module-usage"},{"type":"behaviour","title":"Callbacks - Jido.Action","doc":"Implementing modules must define the following callback:\n\n- `c:run/2`: Executes the main logic of the Action.\n\nOptional callbacks for custom behavior:\n\n- `c:on_before_validate_params/1`: Called before parameter validation.\n- `c:on_after_validate_params/1`: Called after parameter validation.\n- `c:on_after_run/1`: Called after the Action's main logic has executed.","ref":"Jido.Action.html#module-callbacks"},{"type":"behaviour","title":"Error Handling - Jido.Action","doc":"Actions use the `OK` monad for consistent error handling. Errors are wrapped\nin `Jido.Error` structs for uniform error reporting across the system.","ref":"Jido.Action.html#module-error-handling"},{"type":"behaviour","title":"Parameter Validation - Jido.Action","doc":"> **Note on Validation:** The validation process for Actions is intentionally loose.\n> Only fields specified in the schema are validated. Unspecified fields are not\n> validated, allowing for easier Action composition. This approach enables Actions\n> to accept and pass along additional parameters that may be required by other\n> Actions in a chain without causing validation errors.","ref":"Jido.Action.html#module-parameter-validation"},{"type":"macro","title":"Jido.Action.__using__/1","doc":"Defines a new Action module.\n\nThis macro sets up the necessary structure and callbacks for a Action,\nincluding configuration validation and default implementations.","ref":"Jido.Action.html#__using__/1"},{"type":"macro","title":"Options - Jido.Action.__using__/1","doc":"* `:name` - Required. The name of the Action. Must contain only letters, numbers, and underscores.\n\n* `:description` (`t:String.t/0`) - A description of what the Action does.\n\n* `:category` (`t:String.t/0`) - The category of the Action.\n\n* `:tags` (list of `t:String.t/0`) - A list of tags associated with the Action. The default value is `[]`.\n\n* `:vsn` (`t:String.t/0`) - The version of the Action.\n\n* `:compensation` (`t:keyword/0`) - The default value is `[]`.\n\n  * `:enabled` (`t:boolean/0`) - The default value is `false`.\n\n  * `:max_retries` (`t:non_neg_integer/0`) - The default value is `1`.\n\n  * `:timeout` (`t:non_neg_integer/0`) - The default value is `5000`.\n\n* `:schema` (`t:keyword/0`) - A NimbleOptions schema for validating the Action's input parameters. The default value is `[]`.","ref":"Jido.Action.html#__using__/1-options"},{"type":"macro","title":"Examples - Jido.Action.__using__/1","doc":"defmodule MyAction do\n      use Jido.Action,\n        name: \"my_action\",\n        description: \"Performs a specific workflow\",\n        schema: [\n          input: [type: :string, required: true]\n        ]\n\n      @impl true\n      def run(params, _context) do\n        {:ok, %{result: String.upcase(params.input)}}\n      end\n    end","ref":"Jido.Action.html#__using__/1-examples"},{"type":"function","title":"Jido.Action.format_config_error/1","doc":"Formats error messages for action configuration errors.","ref":"Jido.Action.html#format_config_error/1"},{"type":"function","title":"Parameters - Jido.Action.format_config_error/1","doc":"- `error`: The error to format. Can be a `NimbleOptions.ValidationError` or any term.","ref":"Jido.Action.html#format_config_error/1-parameters"},{"type":"function","title":"Returns - Jido.Action.format_config_error/1","doc":"A formatted error message as a string.","ref":"Jido.Action.html#format_config_error/1-returns"},{"type":"function","title":"Examples - Jido.Action.format_config_error/1","doc":"iex> error = %NimbleOptions.ValidationError{keys_path: [:name], message: \"is invalid\"}\n    iex> Jido.Action.format_config_error(error)\n    \"Invalid configuration given to use Jido.Action for key [:name]: is invalid\"","ref":"Jido.Action.html#format_config_error/1-examples"},{"type":"function","title":"Jido.Action.format_validation_error/1","doc":"Formats error messages for action validation errors.","ref":"Jido.Action.html#format_validation_error/1"},{"type":"function","title":"Parameters - Jido.Action.format_validation_error/1","doc":"- `error`: The error to format. Can be a `NimbleOptions.ValidationError` or any term.","ref":"Jido.Action.html#format_validation_error/1-parameters"},{"type":"function","title":"Returns - Jido.Action.format_validation_error/1","doc":"A formatted error message as a string.","ref":"Jido.Action.html#format_validation_error/1-returns"},{"type":"function","title":"Examples - Jido.Action.format_validation_error/1","doc":"iex> error = %NimbleOptions.ValidationError{keys_path: [:input], message: \"is required\"}\n    iex> Jido.Action.format_validation_error(error)\n    \"Invalid parameters for Action at [:input]: is required\"","ref":"Jido.Action.html#format_validation_error/1-examples"},{"type":"function","title":"Jido.Action.new/0","doc":"Raises an error indicating that Actions cannot be defined at runtime.\n\nThis function exists to prevent misuse of the Action system, as Actions\nare designed to be defined at compile-time only.","ref":"Jido.Action.html#new/0"},{"type":"function","title":"Returns - Jido.Action.new/0","doc":"Always returns `{:error, reason}` where `reason` is a config error.","ref":"Jido.Action.html#new/0-returns"},{"type":"function","title":"Examples - Jido.Action.new/0","doc":"iex> Jido.Action.new()\n    {:error, %Jido.Error{type: :config_error, message: \"Actions should not be defined at runtime\"}}","ref":"Jido.Action.html#new/0-examples"},{"type":"function","title":"Jido.Action.new/1","doc":"","ref":"Jido.Action.html#new/1"},{"type":"callback","title":"Jido.Action.on_after_run/1","doc":"Called after the Action's main logic has executed.\n\nThis optional callback allows for post-processing of the Action's result\nbefore it is returned to the caller.","ref":"Jido.Action.html#c:on_after_run/1"},{"type":"callback","title":"Parameters - Jido.Action.on_after_run/1","doc":"- `result`: The result map returned by the `run/2` function.","ref":"Jido.Action.html#c:on_after_run/1-parameters"},{"type":"callback","title":"Returns - Jido.Action.on_after_run/1","doc":"- `{:ok, modified_result}` where `modified_result` is a potentially modified result map.\n- `{:error, reason}` if post-processing fails.","ref":"Jido.Action.html#c:on_after_run/1-returns"},{"type":"callback","title":"Jido.Action.on_after_validate_params/1","doc":"Called after parameter validation.\n\nThis optional callback allows for post-processing of validated parameters\nbefore they are passed to the `run/2` function.","ref":"Jido.Action.html#c:on_after_validate_params/1"},{"type":"callback","title":"Parameters - Jido.Action.on_after_validate_params/1","doc":"- `params`: A map of validated input parameters.","ref":"Jido.Action.html#c:on_after_validate_params/1-parameters"},{"type":"callback","title":"Returns - Jido.Action.on_after_validate_params/1","doc":"- `{:ok, modified_params}` where `modified_params` is a map of potentially modified parameters.\n- `{:error, reason}` if post-processing fails.","ref":"Jido.Action.html#c:on_after_validate_params/1-returns"},{"type":"callback","title":"Jido.Action.on_before_validate_params/1","doc":"Called before parameter validation.\n\nThis optional callback allows for pre-processing of input parameters\nbefore they are validated against the Action's schema.","ref":"Jido.Action.html#c:on_before_validate_params/1"},{"type":"callback","title":"Parameters - Jido.Action.on_before_validate_params/1","doc":"- `params`: A map of raw input parameters.","ref":"Jido.Action.html#c:on_before_validate_params/1-parameters"},{"type":"callback","title":"Returns - Jido.Action.on_before_validate_params/1","doc":"- `{:ok, modified_params}` where `modified_params` is a map of potentially modified parameters.\n- `{:error, reason}` if pre-processing fails.","ref":"Jido.Action.html#c:on_before_validate_params/1-returns"},{"type":"callback","title":"Jido.Action.on_error/4","doc":"Handles errors and performs compensation when enabled.\n\nCalled when an error occurs during Action execution if compensation is enabled\nin the Action's configuration.","ref":"Jido.Action.html#c:on_error/4"},{"type":"callback","title":"Parameters - Jido.Action.on_error/4","doc":"- `failed_params`: The parameters that were passed to the failed execution\n- `error`: The Error struct describing what went wrong\n- `context`: The execution context at the time of failure\n- `opts`: Additional options for compensation handling","ref":"Jido.Action.html#c:on_error/4-parameters"},{"type":"callback","title":"Returns - Jido.Action.on_error/4","doc":"- `{:ok, result}` if compensation succeeded\n- `{:error, reason}` if compensation failed","ref":"Jido.Action.html#c:on_error/4-returns"},{"type":"callback","title":"Examples - Jido.Action.on_error/4","doc":"def on_error(params, error, context, opts) do\n      # Perform compensation logic\n      case rollback_changes(params) do\n        :ok -> {:ok, %{compensated: true, original_error: error}}\n        {:error, reason} -> {:error, \"Compensation failed: #{reason}\"}\n      end\n    end","ref":"Jido.Action.html#c:on_error/4-examples"},{"type":"callback","title":"Jido.Action.run/2","doc":"Executes the Action with the given parameters and context.\n\nThis callback must be implemented by modules using `Jido.Action`.","ref":"Jido.Action.html#c:run/2"},{"type":"callback","title":"Parameters - Jido.Action.run/2","doc":"- `params`: A map of validated input parameters.\n- `context`: A map containing any additional context for the workflow.","ref":"Jido.Action.html#c:run/2-parameters"},{"type":"callback","title":"Returns - Jido.Action.run/2","doc":"- `{:ok, result}` where `result` is a map containing the workflow's output.\n- `{:error, reason}` where `reason` describes why the workflow failed.","ref":"Jido.Action.html#c:run/2-returns"},{"type":"type","title":"Jido.Action.t/0","doc":"","ref":"Jido.Action.html#t:t/0"},{"type":"behaviour","title":"Jido.Agent","doc":"Defines an Agent within the Jido system.\n\nAn Agent represents a higher-level entity that can plan and execute a series of Actions.\nAgents are defined at compile-time and provide a consistent interface for planning,\nexecuting, and managing complex workflows.","ref":"Jido.Agent.html"},{"type":"behaviour","title":"Features - Jido.Agent","doc":"- Compile-time configuration validation\n- Runtime input parameter validation\n- Consistent error handling and formatting\n- Extensible lifecycle hooks\n- JSON serialization support\n- Dynamic planning and execution of Action sequences","ref":"Jido.Agent.html#module-features"},{"type":"behaviour","title":"Usage - Jido.Agent","doc":"To define a new Agent, use the `Jido.Agent` behavior in your module:\n\n    defmodule MyAgent do\n      use Jido.Agent,\n        name: \"my_agent\",\n        description: \"Performs a complex workflow\",\n        category: \"processing\",\n        tags: [\"example\", \"demo\"],\n        vsn: \"1.0.0\",\n        schema: [\n          input: [type: :string, required: true]\n        ]\n\n      @impl true\n      def plan(agent) do\n        # Your planning logic here\n        {:ok, %Jido.ActionSet{agent: agent, plan: [MyAction1, MyAction2]}}\n      end\n    end","ref":"Jido.Agent.html#module-usage"},{"type":"behaviour","title":"Callbacks - Jido.Agent","doc":"Implementing modules must define the following callback:\n\n- `c:plan/1`: Generates a plan (sequence of Actions) for the Agent to execute.","ref":"Jido.Agent.html#module-callbacks"},{"type":"function","title":"Jido.Agent.format_config_error/1","doc":"","ref":"Jido.Agent.html#format_config_error/1"},{"type":"function","title":"Jido.Agent.format_validation_error/1","doc":"","ref":"Jido.Agent.html#format_validation_error/1"},{"type":"callback","title":"Jido.Agent.plan/1","doc":"","ref":"Jido.Agent.html#c:plan/1"},{"type":"type","title":"Jido.Agent.t/0","doc":"","ref":"Jido.Agent.html#t:t/0"},{"type":"behaviour","title":"Jido.Sensor","doc":"Defines the behavior and implementation for Sensors in the Jido system.\n\nA Sensor is a GenServer that emits Signals on PubSub based on specific events and retains a configurable number of last values.","ref":"Jido.Sensor.html"},{"type":"behaviour","title":"Usage - Jido.Sensor","doc":"To define a new Sensor, use the `Jido.Sensor` behavior in your module:\n\n    defmodule MySensor do\n      use Jido.Sensor,\n        name: \"my_sensor\",\n        description: \"Monitors a specific metric\",\n        category: :monitoring,\n        tags: [:example, :demo],\n        vsn: \"1.0.0\",\n        schema: [\n          metric: [type: :string, required: true]\n        ]\n\n      @impl true\n      def generate_signal(state) do\n        # Your sensor logic here\n        {:ok, Jido.Signal.new(%{\n          source: \"#{state.sensor.name}:#{state.id}\",\n          topic: \"metric_update\",\n          payload: %{value: get_metric_value()},\n          timestamp: DateTime.utc_now()\n        })}\n      end\n    end","ref":"Jido.Sensor.html#module-usage"},{"type":"behaviour","title":"Callbacks - Jido.Sensor","doc":"Implementing modules can override the following callbacks:\n\n- `c:mount/1`: Called when the sensor is initialized.\n- `c:generate_signal/1`: Generates a signal based on the current state.\n- `c:before_publish/2`: Called before a signal is published.\n- `c:shutdown/1`: Called when the sensor is shutting down.","ref":"Jido.Sensor.html#module-callbacks"},{"type":"callback","title":"Jido.Sensor.before_publish/2","doc":"","ref":"Jido.Sensor.html#c:before_publish/2"},{"type":"callback","title":"Jido.Sensor.generate_signal/1","doc":"","ref":"Jido.Sensor.html#c:generate_signal/1"},{"type":"callback","title":"Jido.Sensor.mount/1","doc":"","ref":"Jido.Sensor.html#c:mount/1"},{"type":"callback","title":"Jido.Sensor.shutdown/1","doc":"","ref":"Jido.Sensor.html#c:shutdown/1"},{"type":"type","title":"Jido.Sensor.options/0","doc":"","ref":"Jido.Sensor.html#t:options/0"},{"type":"type","title":"Jido.Sensor.t/0","doc":"","ref":"Jido.Sensor.html#t:t/0"},{"type":"module","title":"Jido.Signal","doc":"Defines the structure and behavior of a Signal in the Jido system.\nThis is a local implementation of the CloudEvents specification v1.0.","ref":"Jido.Signal.html"},{"type":"function","title":"Jido.Signal.from_map/1","doc":"Creates a new Signal struct from a map.","ref":"Jido.Signal.html#from_map/1"},{"type":"function","title":"Parameters - Jido.Signal.from_map/1","doc":"- `map`: A map containing the Signal attributes.","ref":"Jido.Signal.html#from_map/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.from_map/1","doc":"`{:ok, Signal.t()}` if the map is valid, `{:error, String.t()}` otherwise.","ref":"Jido.Signal.html#from_map/1-returns"},{"type":"function","title":"Examples - Jido.Signal.from_map/1","doc":"iex> Jido.Signal.from_map(%{\"type\" => \"example.event\", \"source\" => \"/example\", \"id\" => \"123\"})\n    {:ok, %Jido.Signal{type: \"example.event\", source: \"/example\", id: \"123\", ...}}","ref":"Jido.Signal.html#from_map/1-examples"},{"type":"function","title":"Jido.Signal.new/1","doc":"Creates a new Signal struct.","ref":"Jido.Signal.html#new/1"},{"type":"function","title":"Parameters - Jido.Signal.new/1","doc":"- `attrs`: A map containing the Signal attributes.","ref":"Jido.Signal.html#new/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.new/1","doc":"`{:ok, Signal.t()}` if the attributes are valid, `{:error, String.t()}` otherwise.","ref":"Jido.Signal.html#new/1-returns"},{"type":"function","title":"Examples - Jido.Signal.new/1","doc":"iex> Jido.Signal.new(%{type: \"example.event\", source: \"/example\", id: \"123\"})\n    {:ok, %Jido.Signal{type: \"example.event\", source: \"/example\", id: \"123\", ...}}","ref":"Jido.Signal.html#new/1-examples"},{"type":"type","title":"Jido.Signal.t/0","doc":"","ref":"Jido.Signal.html#t:t/0"},{"type":"module","title":"Jido.Workflow","doc":"Workflows are the Action runtime. They provide a robust framework for executing and managing workflows (multiple Actions) in a distributed system.\n\nThis module offers functionality to:\n- Run workflows synchronously or asynchronously\n- Manage timeouts and retries\n- Cancel running workflows\n- Normalize and validate input parameters and context\n- Emit telemetry events for monitoring and debugging\n\nWorkflows are defined as modules (Actions) that implement specific callbacks, allowing for\na standardized way of defining and executing complex workflows across a distributed system.","ref":"Jido.Workflow.html"},{"type":"module","title":"Features - Jido.Workflow","doc":"- Synchronous and asynchronous workflow execution\n- Automatic retries with exponential backoff\n- Timeout handling for long-running workflows\n- Parameter and context normalization\n- Comprehensive error handling and reporting\n- Telemetry integration for monitoring and tracing\n- Cancellation of running workflows","ref":"Jido.Workflow.html#module-features"},{"type":"module","title":"Usage - Jido.Workflow","doc":"Workflows are executed using the `run/4` or `run_async/4` functions:\n\n    Jido.Workflow.run(MyAction, %{param1: \"value\"}, %{context_key: \"context_value\"})\n\nSee `Jido.Action` for how to define a Action.\n\nFor asynchronous execution:\n\n    async_ref = Jido.Workflow.run_async(MyAction, params, context)\n    # ... do other work ...\n    result = Jido.Workflow.await(async_ref)","ref":"Jido.Workflow.html#module-usage"},{"type":"function","title":"Jido.Workflow.await/2","doc":"Waits for the result of an asynchronous Action execution.","ref":"Jido.Workflow.html#await/2"},{"type":"function","title":"Parameters - Jido.Workflow.await/2","doc":"- `async_ref`: The reference returned by `run_async/4`.\n- `timeout`: Maximum time (in ms) to wait for the result (default: 5000).","ref":"Jido.Workflow.html#await/2-parameters"},{"type":"function","title":"Returns - Jido.Workflow.await/2","doc":"- `{:ok, result}` if the Action executes successfully.\n- `{:error, reason}` if an error occurs during execution or if the workflow times out.","ref":"Jido.Workflow.html#await/2-returns"},{"type":"function","title":"Examples - Jido.Workflow.await/2","doc":"iex> async_ref = Jido.Workflow.run_async(MyAction, %{input: \"value\"})\n    iex> Jido.Workflow.await(async_ref, 10_000)\n    {:ok, %{result: \"processed value\"}}\n\n    iex> async_ref = Jido.Workflow.run_async(SlowAction, %{input: \"value\"})\n    iex> Jido.Workflow.await(async_ref, 100)\n    {:error, %Jido.Error{type: :timeout, message: \"Async workflow timed out after 100ms\"}}","ref":"Jido.Workflow.html#await/2-examples"},{"type":"function","title":"Jido.Workflow.cancel/1","doc":"Cancels a running asynchronous Action execution.","ref":"Jido.Workflow.html#cancel/1"},{"type":"function","title":"Parameters - Jido.Workflow.cancel/1","doc":"- `async_ref`: The reference returned by `run_async/4`, or just the PID of the process to cancel.","ref":"Jido.Workflow.html#cancel/1-parameters"},{"type":"function","title":"Returns - Jido.Workflow.cancel/1","doc":"- `:ok` if the cancellation was successful.\n- `{:error, reason}` if the cancellation failed or the input was invalid.","ref":"Jido.Workflow.html#cancel/1-returns"},{"type":"function","title":"Examples - Jido.Workflow.cancel/1","doc":"iex> async_ref = Jido.Workflow.run_async(LongRunningAction, %{input: \"value\"})\n    iex> Jido.Workflow.cancel(async_ref)\n    :ok\n\n    iex> Jido.Workflow.cancel(\"invalid\")\n    {:error, %Jido.Error{type: :invalid_async_ref, message: \"Invalid async ref for cancellation\"}}","ref":"Jido.Workflow.html#cancel/1-examples"},{"type":"function","title":"Jido.Workflow.run/4","doc":"Executes a Action synchronously with the given parameters and context.","ref":"Jido.Workflow.html#run/4"},{"type":"function","title":"Parameters - Jido.Workflow.run/4","doc":"- `action`: The module implementing the Action behavior.\n- `params`: A map of input parameters for the Action.\n- `context`: A map providing additional context for the Action execution.\n- `opts`: Options controlling the execution:\n  - `:timeout` - Maximum time (in ms) allowed for the Action to complete (default: 5000).\n  - `:max_retries` - Maximum number of retry attempts (default: 1).\n  - `:backoff` - Initial backoff time in milliseconds, doubles with each retry (default: 1000).","ref":"Jido.Workflow.html#run/4-parameters"},{"type":"function","title":"Returns - Jido.Workflow.run/4","doc":"- `{:ok, result}` if the Action executes successfully.\n- `{:error, reason}` if an error occurs during execution.","ref":"Jido.Workflow.html#run/4-returns"},{"type":"function","title":"Examples - Jido.Workflow.run/4","doc":"iex> Jido.Workflow.run(MyAction, %{input: \"value\"}, %{user_id: 123})\n    {:ok, %{result: \"processed value\"}}\n\n    iex> Jido.Workflow.run(MyAction, %{invalid: \"input\"}, %{}, timeout: 1000)\n    {:error, %Jido.Error{type: :validation_error, message: \"Invalid input\"}}","ref":"Jido.Workflow.html#run/4-examples"},{"type":"function","title":"Jido.Workflow.run_async/4","doc":"Executes a Action asynchronously with the given parameters and context.\n\nThis function immediately returns a reference that can be used to await the result\nor cancel the workflow.","ref":"Jido.Workflow.html#run_async/4"},{"type":"function","title":"Parameters - Jido.Workflow.run_async/4","doc":"- `action`: The module implementing the Action behavior.\n- `params`: A map of input parameters for the Action.\n- `context`: A map providing additional context for the Action execution.\n- `opts`: Options controlling the execution (same as `run/4`).","ref":"Jido.Workflow.html#run_async/4-parameters"},{"type":"function","title":"Returns - Jido.Workflow.run_async/4","doc":"An `async_ref` map containing:\n- `:ref` - A unique reference for this async workflow.\n- `:pid` - The PID of the process executing the Action.","ref":"Jido.Workflow.html#run_async/4-returns"},{"type":"function","title":"Examples - Jido.Workflow.run_async/4","doc":"iex> async_ref = Jido.Workflow.run_async(MyAction, %{input: \"value\"}, %{user_id: 123})\n    %{ref: #Reference<0.1234.5678>, pid: #PID<0.234.0>}\n\n    iex> result = Jido.Workflow.await(async_ref)\n    {:ok, %{result: \"processed value\"}}","ref":"Jido.Workflow.html#run_async/4-examples"},{"type":"type","title":"Jido.Workflow.action/0","doc":"","ref":"Jido.Workflow.html#t:action/0"},{"type":"type","title":"Jido.Workflow.async_ref/0","doc":"","ref":"Jido.Workflow.html#t:async_ref/0"},{"type":"type","title":"Jido.Workflow.context/0","doc":"","ref":"Jido.Workflow.html#t:context/0"},{"type":"type","title":"Jido.Workflow.params/0","doc":"","ref":"Jido.Workflow.html#t:params/0"},{"type":"type","title":"Jido.Workflow.run_opts/0","doc":"","ref":"Jido.Workflow.html#t:run_opts/0"},{"type":"module","title":"Jido.Workflow.Chain","doc":"Provides functionality to chain multiple Jido Workflows together with interruption support.\n\nThis module allows for sequential execution of workflows, where the output\nof one workflow becomes the input for the next workflow in the chain.\nExecution can be interrupted between workflows using an interruption check function.","ref":"Jido.Workflow.Chain.html"},{"type":"function","title":"Jido.Workflow.Chain.chain/3","doc":"Executes a chain of workflows sequentially with optional interruption support.","ref":"Jido.Workflow.Chain.html#chain/3"},{"type":"function","title":"Parameters - Jido.Workflow.Chain.chain/3","doc":"- `workflows`: A list of workflows to be executed in order. Each workflow\n  can be a module (the workflow module) or a tuple of `{workflow_module, options}`.\n- `initial_params`: A map of initial parameters to be passed to the first workflow.\n- `opts`: Additional options for the chain execution.","ref":"Jido.Workflow.Chain.html#chain/3-parameters"},{"type":"function","title":"Options - Jido.Workflow.Chain.chain/3","doc":"- `:async` - When set to `true`, the chain will be executed asynchronously (default: `false`).\n- `:context` - A map of context data to be passed to each workflow.\n- `:interrupt_check` - A function that returns boolean, called between workflows to check if chain should be interrupted.","ref":"Jido.Workflow.Chain.html#chain/3-options"},{"type":"function","title":"Returns - Jido.Workflow.Chain.chain/3","doc":"- `{:ok, result}` where `result` is the final output of the chain.\n- `{:error, error}` if any workflow in the chain fails.\n- `{:interrupted, result}` if the chain was interrupted, containing the last successful result.\n- `Task.t()` if the `:async` option is set to `true`.","ref":"Jido.Workflow.Chain.html#chain/3-returns"},{"type":"function","title":"Examples - Jido.Workflow.Chain.chain/3","doc":"iex> interrupt_check = fn -> System.monotonic_time(:millisecond) > @deadline end\n    iex> Jido.Workflow.Chain.chain([AddOne, MultiplyByTwo], %{value: 5}, interrupt_check: interrupt_check)\n    {:ok, %{value: 12}}\n\n    # When interrupted:\n    iex> Jido.Workflow.Chain.chain([AddOne, MultiplyByTwo], %{value: 5}, interrupt_check: fn -> true end)\n    {:interrupted, %{value: 6}}","ref":"Jido.Workflow.Chain.html#chain/3-examples"},{"type":"type","title":"Jido.Workflow.Chain.chain_result/0","doc":"","ref":"Jido.Workflow.Chain.html#t:chain_result/0"},{"type":"type","title":"Jido.Workflow.Chain.chain_workflow/0","doc":"","ref":"Jido.Workflow.Chain.html#t:chain_workflow/0"},{"type":"type","title":"Jido.Workflow.Chain.interrupt_check/0","doc":"","ref":"Jido.Workflow.Chain.html#t:interrupt_check/0"},{"type":"module","title":"Jido.Workflow.Closure","doc":"Provides functionality to create closures around Jido Workflows (Actions).\n\nThis module allows for partial application of context and options to actions,\ncreating reusable workflow closures that can be executed with different parameters.","ref":"Jido.Workflow.Closure.html"},{"type":"function","title":"Jido.Workflow.Closure.async_closure/3","doc":"Creates an async closure around a action with pre-applied context and options.","ref":"Jido.Workflow.Closure.html#async_closure/3"},{"type":"function","title":"Parameters - Jido.Workflow.Closure.async_closure/3","doc":"- `action`: The action module to create an async closure for.\n- `context`: The context to be applied to the action (default: %{}).\n- `opts`: The options to be applied to the action execution (default: []).","ref":"Jido.Workflow.Closure.html#async_closure/3-parameters"},{"type":"function","title":"Returns - Jido.Workflow.Closure.async_closure/3","doc":"A function that takes params and returns an async reference.","ref":"Jido.Workflow.Closure.html#async_closure/3-returns"},{"type":"function","title":"Examples - Jido.Workflow.Closure.async_closure/3","doc":"iex> async_closure = Jido.Workflow.Closure.async_closure(MyAction, %{user_id: 123}, [timeout: 10_000])\n    iex> async_ref = async_closure.(%{input: \"test\"})\n    iex> Jido.Workflow.await(async_ref)\n    {:ok, %{result: \"processed test\"}}","ref":"Jido.Workflow.Closure.html#async_closure/3-examples"},{"type":"function","title":"Jido.Workflow.Closure.closure/3","doc":"Creates a closure around a action with pre-applied context and options.","ref":"Jido.Workflow.Closure.html#closure/3"},{"type":"function","title":"Parameters - Jido.Workflow.Closure.closure/3","doc":"- `action`: The action module to create a closure for.\n- `context`: The context to be applied to the action (default: %{}).\n- `opts`: The options to be applied to the action execution (default: []).","ref":"Jido.Workflow.Closure.html#closure/3-parameters"},{"type":"function","title":"Returns - Jido.Workflow.Closure.closure/3","doc":"A function that takes params and returns the result of running the action.","ref":"Jido.Workflow.Closure.html#closure/3-returns"},{"type":"function","title":"Examples - Jido.Workflow.Closure.closure/3","doc":"iex> closure = Jido.Workflow.Closure.closure(MyAction, %{user_id: 123}, [timeout: 10_000])\n    iex> closure.(%{input: \"test\"})\n    {:ok, %{result: \"processed test\"}}","ref":"Jido.Workflow.Closure.html#closure/3-examples"},{"type":"type","title":"Jido.Workflow.Closure.action/0","doc":"","ref":"Jido.Workflow.Closure.html#t:action/0"},{"type":"type","title":"Jido.Workflow.Closure.closure/0","doc":"","ref":"Jido.Workflow.Closure.html#t:closure/0"},{"type":"type","title":"Jido.Workflow.Closure.context/0","doc":"","ref":"Jido.Workflow.Closure.html#t:context/0"},{"type":"type","title":"Jido.Workflow.Closure.params/0","doc":"","ref":"Jido.Workflow.Closure.html#t:params/0"},{"type":"type","title":"Jido.Workflow.Closure.run_opts/0","doc":"","ref":"Jido.Workflow.Closure.html#t:run_opts/0"},{"type":"module","title":"Jido.Workflow.Tool","doc":"Provides functionality to convert Jido Workflows into tool representations.\n\nThis module allows Jido Workflows to be easily integrated with AI systems\nlike LangChain or Instructor by converting them into a standardized tool format.","ref":"Jido.Workflow.Tool.html"},{"type":"function","title":"Jido.Workflow.Tool.build_parameters_schema/1","doc":"Builds a parameters schema for the tool based on the workflow's schema.","ref":"Jido.Workflow.Tool.html#build_parameters_schema/1"},{"type":"function","title":"Arguments - Jido.Workflow.Tool.build_parameters_schema/1","doc":"* `schema` - The NimbleOptions schema from the workflow.","ref":"Jido.Workflow.Tool.html#build_parameters_schema/1-arguments"},{"type":"function","title":"Returns - Jido.Workflow.Tool.build_parameters_schema/1","doc":"A map representing the parameters schema in a format compatible with LangChain.","ref":"Jido.Workflow.Tool.html#build_parameters_schema/1-returns"},{"type":"function","title":"Jido.Workflow.Tool.execute_workflow/3","doc":"Executes an workflow and formats the result for tool output.\n\nThis function is typically used as the function value in the tool representation.","ref":"Jido.Workflow.Tool.html#execute_workflow/3"},{"type":"function","title":"Jido.Workflow.Tool.nimble_type_to_json_schema_type/1","doc":"Converts a NimbleOptions type to a JSON Schema type.","ref":"Jido.Workflow.Tool.html#nimble_type_to_json_schema_type/1"},{"type":"function","title":"Arguments - Jido.Workflow.Tool.nimble_type_to_json_schema_type/1","doc":"* `type` - The NimbleOptions type.","ref":"Jido.Workflow.Tool.html#nimble_type_to_json_schema_type/1-arguments"},{"type":"function","title":"Returns - Jido.Workflow.Tool.nimble_type_to_json_schema_type/1","doc":"A string representing the equivalent JSON Schema type.","ref":"Jido.Workflow.Tool.html#nimble_type_to_json_schema_type/1-returns"},{"type":"function","title":"Jido.Workflow.Tool.parameter_to_json_schema/1","doc":"Converts a NimbleOptions parameter definition to a JSON Schema representation.","ref":"Jido.Workflow.Tool.html#parameter_to_json_schema/1"},{"type":"function","title":"Arguments - Jido.Workflow.Tool.parameter_to_json_schema/1","doc":"* `opts` - The options for a single parameter from the NimbleOptions schema.","ref":"Jido.Workflow.Tool.html#parameter_to_json_schema/1-arguments"},{"type":"function","title":"Returns - Jido.Workflow.Tool.parameter_to_json_schema/1","doc":"A map representing the parameter in JSON Schema format.","ref":"Jido.Workflow.Tool.html#parameter_to_json_schema/1-returns"},{"type":"function","title":"Jido.Workflow.Tool.to_tool/1","doc":"Converts a Jido Workflow into a tool representation.","ref":"Jido.Workflow.Tool.html#to_tool/1"},{"type":"function","title":"Arguments - Jido.Workflow.Tool.to_tool/1","doc":"* `workflow` - The module implementing the Jido.Action behavior.","ref":"Jido.Workflow.Tool.html#to_tool/1-arguments"},{"type":"function","title":"Returns - Jido.Workflow.Tool.to_tool/1","doc":"A map representing the workflow as a tool, compatible with systems like LangChain.","ref":"Jido.Workflow.Tool.html#to_tool/1-returns"},{"type":"function","title":"Examples - Jido.Workflow.Tool.to_tool/1","doc":"iex> tool = Jido.Workflow.Tool.to_tool(MyWorkflow)\n    %{\n      name: \"my_workflow\",\n      description: \"Performs a specific task\",\n      function: #Function<...>,\n      parameters_schema: %{...}\n    }","ref":"Jido.Workflow.Tool.html#to_tool/1-examples"},{"type":"type","title":"Jido.Workflow.Tool.tool/0","doc":"","ref":"Jido.Workflow.Tool.html#t:tool/0"},{"type":"module","title":"Jido.Agent.Supervisor","doc":"A dynamic supervisor that manages agent workers and their associated processes.\n\nEach agent worker runs under this supervisor and can dynamically start/stop\nadditional child processes. The supervisor provides:\n\n- Dynamic agent worker management\n- Named process registration via Registry\n- Child process supervision\n- Telemetry instrumentation\n- Configurable restart strategies","ref":"Jido.Agent.Supervisor.html"},{"type":"function","title":"Jido.Agent.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Jido.Agent.Supervisor.html#child_spec/1"},{"type":"function","title":"Jido.Agent.Supervisor.find_agent/1","doc":"","ref":"Jido.Agent.Supervisor.html#find_agent/1"},{"type":"function","title":"Jido.Agent.Supervisor.start_agent/2","doc":"","ref":"Jido.Agent.Supervisor.html#start_agent/2"},{"type":"function","title":"Jido.Agent.Supervisor.start_child/2","doc":"","ref":"Jido.Agent.Supervisor.html#start_child/2"},{"type":"function","title":"Jido.Agent.Supervisor.start_link/1","doc":"","ref":"Jido.Agent.Supervisor.html#start_link/1"},{"type":"function","title":"Jido.Agent.Supervisor.terminate_child/1","doc":"","ref":"Jido.Agent.Supervisor.html#terminate_child/1"},{"type":"function","title":"Jido.Agent.Supervisor.which_children/0","doc":"","ref":"Jido.Agent.Supervisor.html#which_children/0"},{"type":"type","title":"Jido.Agent.Supervisor.child_spec/0","doc":"","ref":"Jido.Agent.Supervisor.html#t:child_spec/0"},{"type":"type","title":"Jido.Agent.Supervisor.init_opts/0","doc":"","ref":"Jido.Agent.Supervisor.html#t:init_opts/0"},{"type":"type","title":"Jido.Agent.Supervisor.supervisor_opts/0","doc":"","ref":"Jido.Agent.Supervisor.html#t:supervisor_opts/0"},{"type":"module","title":"Jido.Agent.Worker","doc":"A GenServer implementation for managing Jido agents with centralized command handling\nand pluggable communication.\n\nThis module provides a robust worker process for Jido agents, handling:\n- Agent state management\n- Command processing and queueing\n- PubSub-based communication\n- Metrics and signal emission\n\nIt supports various commands like replan, pause, resume, reset, and stop,\nand manages the agent's lifecycle through different states (idle, planning, running, paused).","ref":"Jido.Agent.Worker.html"},{"type":"function","title":"Jido.Agent.Worker.act/2","doc":"Triggers the agent to act based on its current state.","ref":"Jido.Agent.Worker.html#act/2"},{"type":"function","title":"Parameters - Jido.Agent.Worker.act/2","doc":"* `server` - The GenServer reference\n  * `attrs` - A map of additional attributes for the action","ref":"Jido.Agent.Worker.html#act/2-parameters"},{"type":"function","title":"Returns - Jido.Agent.Worker.act/2","doc":"* `:ok`","ref":"Jido.Agent.Worker.html#act/2-returns"},{"type":"function","title":"Jido.Agent.Worker.child_spec/1","doc":"Returns a child specification for starting the Agent Worker under a supervisor.","ref":"Jido.Agent.Worker.html#child_spec/1"},{"type":"function","title":"Jido.Agent.Worker.cmd/3","doc":"Sends a command to the agent.","ref":"Jido.Agent.Worker.html#cmd/3"},{"type":"function","title":"Parameters - Jido.Agent.Worker.cmd/3","doc":"* `server` - The GenServer reference\n  * `command` - The command to execute\n  * `args` - Additional arguments for the command (optional)","ref":"Jido.Agent.Worker.html#cmd/3-parameters"},{"type":"function","title":"Returns - Jido.Agent.Worker.cmd/3","doc":"* `{:ok, State.t()}` on success\n  * `{:error, term()}` on failure","ref":"Jido.Agent.Worker.html#cmd/3-returns"},{"type":"function","title":"Jido.Agent.Worker.set/2","doc":"Updates the agent's attributes.","ref":"Jido.Agent.Worker.html#set/2"},{"type":"function","title":"Parameters - Jido.Agent.Worker.set/2","doc":"* `server` - The GenServer reference\n  * `attrs` - A map of attributes to update","ref":"Jido.Agent.Worker.html#set/2-parameters"},{"type":"function","title":"Returns - Jido.Agent.Worker.set/2","doc":"* `:ok`","ref":"Jido.Agent.Worker.html#set/2-returns"},{"type":"function","title":"Jido.Agent.Worker.start_link/1","doc":"Starts a new Agent Worker process.","ref":"Jido.Agent.Worker.html#start_link/1"},{"type":"function","title":"Options - Jido.Agent.Worker.start_link/1","doc":"* `:agent` - The Jido agent to manage (required)\n  * `:name` - The name to register the process under (optional, defaults to agent ID)\n  * `:pubsub` - The PubSub module to use for communication (required)","ref":"Jido.Agent.Worker.html#start_link/1-options"},{"type":"function","title":"Returns - Jido.Agent.Worker.start_link/1","doc":"* `{:ok, pid}` on success\n  * `{:error, reason}` on failure","ref":"Jido.Agent.Worker.html#start_link/1-returns"},{"type":"type","title":"Jido.Agent.Worker.agent/0","doc":"","ref":"Jido.Agent.Worker.html#t:agent/0"},{"type":"type","title":"Jido.Agent.Worker.command/0","doc":"","ref":"Jido.Agent.Worker.html#t:command/0"},{"type":"type","title":"Jido.Agent.Worker.topic/0","doc":"","ref":"Jido.Agent.Worker.html#t:topic/0"},{"type":"module","title":"Jido.Actions.Arithmetic","doc":"Provides basic arithmetic operations as actions.\n\nThis module defines a set of arithmetic actions that can be used in workflows:\n- Add: Adds two numbers together\n- Subtract: Subtracts one number from another\n- Multiply: Multiplies two numbers\n- Divide: Divides one number by another, handling division by zero\n- Square: Squares a number\n\nEach action is implemented as a separate submodule and follows the Jido.Action behavior.","ref":"Jido.Actions.Arithmetic.html"},{"type":"module","title":"Jido.Actions.Basic","doc":"A collection of basic actions for common workflows.\n\nThis module provides a set of simple, reusable actions:\n- Sleep: Pauses execution for a specified duration\n- Log: Logs a message with a specified level\n- Todo: Logs a todo item as a placeholder or reminder\n- RandomSleep: Introduces a random delay within a specified range\n- Increment: Increments a value by 1\n- Decrement: Decrements a value by 1\n\nEach action is implemented as a separate submodule and follows the Jido.Action behavior.","ref":"Jido.Actions.Basic.html"},{"type":"module","title":"Jido.Actions.Files","doc":"Actions for file-related operations in workflows.\n\nThis module provides a set of actions for common file operations:\n- WriteFile: Writes content to a file\n- ReadFile: Reads content from a file\n- CopyFile: Copies a file from source to destination\n- MoveFile: Moves a file from source to destination\n\nEach action is implemented as a separate submodule and follows the Jido.Action behavior.","ref":"Jido.Actions.Files.html"},{"type":"module","title":"Jido.Actions.Simplebot","doc":"A collection of actions for a simple robot simulation.\n\nThis module provides actions for:\n- Move: Simulates moving the robot to a specified location\n- Idle: Simulates the robot idling\n- DoWork: Simulates the robot performing work tasks\n- Report: Simulates the robot reporting its status\n- Recharge: Simulates recharging the robot's battery","ref":"Jido.Actions.Simplebot.html"},{"type":"module","title":"Jido.ActionSet","doc":"Encapsulates the planning state and results for an agent.\n\nAn ActionSet represents the complete lifecycle of an agent's planned actions,\nincluding the initial state, the planned actions, execution context, results,\nand any errors that may occur during the process.","ref":"Jido.ActionSet.html"},{"type":"module","title":"Fields - Jido.ActionSet","doc":"* `agent` - The agent state used for planning\n* `plan` - The generated execution plan, consisting of action modules or tuples of {module, options}\n* `context` - Additional context information for action execution\n* `result` - The results of executing the plan\n* `error` - Any errors that occurred during planning or execution","ref":"Jido.ActionSet.html#module-fields"},{"type":"type","title":"Jido.ActionSet.action/0","doc":"A single action in the plan, either a module or a tuple of {module, keyword options}","ref":"Jido.ActionSet.html#t:action/0"},{"type":"type","title":"Jido.ActionSet.agent/0","doc":"Represents an agent's state, typically a struct","ref":"Jido.ActionSet.html#t:agent/0"},{"type":"type","title":"Jido.ActionSet.t/0","doc":"The complete ActionSet structure","ref":"Jido.ActionSet.html#t:t/0"},{"type":"module","title":"Jido.Agent.Worker.State","doc":"Struct module for the Agent Worker state.","ref":"Jido.Agent.Worker.State.html"},{"type":"function","title":"Jido.Agent.Worker.State.default_topics/1","doc":"","ref":"Jido.Agent.Worker.State.html#default_topics/1"},{"type":"type","title":"Jido.Agent.Worker.State.t/0","doc":"","ref":"Jido.Agent.Worker.State.html#t:t/0"},{"type":"module","title":"Jido.Error","doc":"Defines error structures and helper functions for Jido\n\nThis module provides a standardized way to create and handle errors within the Jido system.\nIt offers a set of predefined error types and functions to create, manipulate, and convert\nerror structures consistently across the application.\n\n> Why not use Exceptions?\n>\n> Jido is designed to be a functional system, strictly adhering to the use of result tuples.\n> This approach provides several benefits:\n>\n> 1. Consistent error handling: By using `{:ok, result}` or `{:error, reason}` tuples,\n>    we ensure a uniform way of handling success and failure cases throughout the system.\n>\n> 2. Composability: Monadic workflows can be easily chained together, allowing for\n>    cleaner and more maintainable code.\n>\n> 3. Explicit error paths: The use of result tuples makes error cases explicit,\n>    reducing the likelihood of unhandled errors.\n>\n> 4. No silent failures: Unlike exceptions, which can be silently caught and ignored,\n>    result tuples require explicit handling of both success and error cases.\n>\n> 5. Better testability: Monadic workflows are easier to test, as both success and\n>    error paths can be explicitly verified.\n>\n> By using this approach instead of exceptions, we gain more control over the flow of our\n> workflows and ensure that errors are handled consistently across the entire system.","ref":"Jido.Error.html"},{"type":"module","title":"Usage - Jido.Error","doc":"Use this module to create specific error types when exceptions occur in your Jido workflows.\nThis allows for consistent error handling and reporting throughout the system.\n\nExample:\n\n    defmodule MyWorkflow do\n      alias Jido.Error\n\n      def run(params) do\n        case validate(params) do\n          :ok -> perform_workflow(params)\n          {:error, reason} -> Error.validation_error(\"Invalid parameters\")\n        end\n      end\n    end","ref":"Jido.Error.html#module-usage"},{"type":"function","title":"Jido.Error.bad_request/3","doc":"Creates a new bad request error.\n\nUse this when the client sends an invalid or malformed request.","ref":"Jido.Error.html#bad_request/3"},{"type":"function","title":"Parameters - Jido.Error.bad_request/3","doc":"- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#bad_request/3-parameters"},{"type":"function","title":"Example - Jido.Error.bad_request/3","doc":"iex> Jido.Error.bad_request(\"Missing required parameter 'user_id'\")\n    %Jido.Error{\n      type: :bad_request,\n      message: \"Missing required parameter 'user_id'\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#bad_request/3-example"},{"type":"function","title":"Jido.Error.capture_stacktrace/0","doc":"Captures the current stacktrace.\n\nThis function is useful when you want to capture the stacktrace at a specific point\nin your code, rather than at the point where the error is created. It drops the first\ntwo entries from the stacktrace to remove internal function calls related to this module.","ref":"Jido.Error.html#capture_stacktrace/0"},{"type":"function","title":"Returns - Jido.Error.capture_stacktrace/0","doc":"The current stacktrace as a list.","ref":"Jido.Error.html#capture_stacktrace/0-returns"},{"type":"function","title":"Example - Jido.Error.capture_stacktrace/0","doc":"iex> stacktrace = Jido.Error.capture_stacktrace()\n    iex> is_list(stacktrace)\n    true","ref":"Jido.Error.html#capture_stacktrace/0-example"},{"type":"function","title":"Jido.Error.compensation_error/3","doc":"Creates a new compensation error with details about the original error and compensation attempt.","ref":"Jido.Error.html#compensation_error/3"},{"type":"function","title":"Parameters - Jido.Error.compensation_error/3","doc":"- `original_error`: The error that triggered compensation\n- `details`: Optional map containing:\n  - `:compensated` - Boolean indicating if compensation succeeded\n  - `:compensation_result` - Result from successful compensation\n  - `:compensation_error` - Error from failed compensation\n- `stacktrace`: Optional stacktrace for debugging","ref":"Jido.Error.html#compensation_error/3-parameters"},{"type":"function","title":"Examples - Jido.Error.compensation_error/3","doc":"iex> original_error = Jido.Error.execution_error(\"Failed to process payment\")\n    iex> Jido.Error.compensation_error(original_error, %{\n    ...>   compensated: true,\n    ...>   compensation_result: %{refund_id: \"ref_123\"}\n    ...> })\n    %Jido.Error{\n      type: :compensation_error,\n      message: \"Compensation completed for: Failed to process payment\",\n      details: %{\n        compensated: true,\n        compensation_result: %{refund_id: \"ref_123\"},\n        original_error: %Jido.Error{...}\n      }\n    }\n\n    iex> # For failed compensation:\n    iex> Jido.Error.compensation_error(original_error, %{\n    ...>   compensated: false,\n    ...>   compensation_error: \"Refund failed\"\n    ...> })","ref":"Jido.Error.html#compensation_error/3-examples"},{"type":"function","title":"Jido.Error.config_error/3","doc":"Creates a new config error.\n\nUse this when there's an issue with the system or workflow configuration.","ref":"Jido.Error.html#config_error/3"},{"type":"function","title":"Parameters - Jido.Error.config_error/3","doc":"- `message`: A string describing the configuration error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#config_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.config_error/3","doc":"iex> Jido.Error.config_error(\"Invalid database connection string\")\n    %Jido.Error{\n      type: :config_error,\n      message: \"Invalid database connection string\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#config_error/3-example"},{"type":"function","title":"Jido.Error.execution_error/3","doc":"Creates a new execution error.\n\nUse this when an error occurs during the execution of an workflow.","ref":"Jido.Error.html#execution_error/3"},{"type":"function","title":"Parameters - Jido.Error.execution_error/3","doc":"- `message`: A string describing the execution error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#execution_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.execution_error/3","doc":"iex> Jido.Error.execution_error(\"Failed to process data\", %{step: \"data_transformation\"})\n    %Jido.Error{\n      type: :execution_error,\n      message: \"Failed to process data\",\n      details: %{step: \"data_transformation\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#execution_error/3-example"},{"type":"function","title":"Jido.Error.internal_server_error/3","doc":"Creates a new internal server error.\n\nUse this for unexpected errors that occur within the system.","ref":"Jido.Error.html#internal_server_error/3"},{"type":"function","title":"Parameters - Jido.Error.internal_server_error/3","doc":"- `message`: A string describing the internal server error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#internal_server_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.internal_server_error/3","doc":"iex> Jido.Error.internal_server_error(\"Unexpected error in data processing\")\n    %Jido.Error{\n      type: :internal_server_error,\n      message: \"Unexpected error in data processing\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#internal_server_error/3-example"},{"type":"function","title":"Jido.Error.invalid_action/3","doc":"Creates a new invalid action error.\n\nUse this when a action is improperly defined or used within the Jido system.","ref":"Jido.Error.html#invalid_action/3"},{"type":"function","title":"Parameters - Jido.Error.invalid_action/3","doc":"- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#invalid_action/3-parameters"},{"type":"function","title":"Example - Jido.Error.invalid_action/3","doc":"iex> Jido.Error.invalid_action(\"Action 'MyAction' is missing required callback\")\n    %Jido.Error{\n      type: :invalid_action,\n      message: \"Action 'MyAction' is missing required callback\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#invalid_action/3-example"},{"type":"function","title":"Jido.Error.invalid_async_ref/3","doc":"Creates a new invalid async ref error.\n\nUse this when an invalid reference to an asynchronous workflow is encountered.","ref":"Jido.Error.html#invalid_async_ref/3"},{"type":"function","title":"Parameters - Jido.Error.invalid_async_ref/3","doc":"- `message`: A string describing the invalid async ref error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#invalid_async_ref/3-parameters"},{"type":"function","title":"Example - Jido.Error.invalid_async_ref/3","doc":"iex> Jido.Error.invalid_async_ref(\"Invalid or expired async workflow reference\")\n    %Jido.Error{\n      type: :invalid_async_ref,\n      message: \"Invalid or expired async workflow reference\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#invalid_async_ref/3-example"},{"type":"function","title":"Jido.Error.invalid_sensor/3","doc":"Creates a new invalid sensor error.\n\nUse this when a sensor is improperly defined or used within the Jido system.","ref":"Jido.Error.html#invalid_sensor/3"},{"type":"function","title":"Parameters - Jido.Error.invalid_sensor/3","doc":"- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#invalid_sensor/3-parameters"},{"type":"function","title":"Example - Jido.Error.invalid_sensor/3","doc":"iex> Jido.Error.invalid_sensor(\"Sensor 'MySensor' is missing required callback\")\n    %Jido.Error{\n      type: :invalid_sensor,\n      message: \"Sensor 'MySensor' is missing required callback\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#invalid_sensor/3-example"},{"type":"function","title":"Jido.Error.new/4","doc":"Creates a new error struct with the given type and message.\n\nThis is a low-level function used by other error creation functions in this module.\nConsider using the specific error creation functions unless you need fine-grained control.","ref":"Jido.Error.html#new/4"},{"type":"function","title":"Parameters - Jido.Error.new/4","doc":"- `type`: The error type (see `t:error_type/0`).\n- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#new/4-parameters"},{"type":"function","title":"Examples - Jido.Error.new/4","doc":"iex> Jido.Error.new(:config_error, \"Invalid configuration\")\n    %Jido.Error{\n      type: :config_error,\n      message: \"Invalid configuration\",\n      details: nil,\n      stacktrace: [...]\n    }\n\n    iex> Jido.Error.new(:execution_error, \"Workflow failed\", %{step: \"data_processing\"})\n    %Jido.Error{\n      type: :execution_error,\n      message: \"Workflow failed\",\n      details: %{step: \"data_processing\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#new/4-examples"},{"type":"function","title":"Jido.Error.timeout/3","doc":"Creates a new timeout error.\n\nUse this when an workflow exceeds its allocated time limit.","ref":"Jido.Error.html#timeout/3"},{"type":"function","title":"Parameters - Jido.Error.timeout/3","doc":"- `message`: A string describing the timeout error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#timeout/3-parameters"},{"type":"function","title":"Example - Jido.Error.timeout/3","doc":"iex> Jido.Error.timeout(\"Workflow timed out after 30 seconds\", %{workflow: \"FetchUserData\"})\n    %Jido.Error{\n      type: :timeout,\n      message: \"Workflow timed out after 30 seconds\",\n      details: %{workflow: \"FetchUserData\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#timeout/3-example"},{"type":"function","title":"Jido.Error.to_map/1","doc":"Converts the error struct to a plain map.\n\nThis function transforms the error struct into a plain map,\nincluding the error type and stacktrace if available. It's useful\nfor serialization or when working with APIs that expect plain maps.","ref":"Jido.Error.html#to_map/1"},{"type":"function","title":"Parameters - Jido.Error.to_map/1","doc":"- `error`: An error struct of type `t:t/0`.","ref":"Jido.Error.html#to_map/1-parameters"},{"type":"function","title":"Returns - Jido.Error.to_map/1","doc":"A map representation of the error.","ref":"Jido.Error.html#to_map/1-returns"},{"type":"function","title":"Example - Jido.Error.to_map/1","doc":"iex> error = Jido.Error.validation_error(\"Invalid input\")\n    iex> Jido.Error.to_map(error)\n    %{\n      type: :validation_error,\n      message: \"Invalid input\",\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#to_map/1-example"},{"type":"function","title":"Jido.Error.validation_error/3","doc":"Creates a new validation error.\n\nUse this when input validation fails for an workflow.","ref":"Jido.Error.html#validation_error/3"},{"type":"function","title":"Parameters - Jido.Error.validation_error/3","doc":"- `message`: A string describing the validation error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#validation_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.validation_error/3","doc":"iex> Jido.Error.validation_error(\"Invalid email format\", %{field: \"email\", value: \"not-an-email\"})\n    %Jido.Error{\n      type: :validation_error,\n      message: \"Invalid email format\",\n      details: %{field: \"email\", value: \"not-an-email\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#validation_error/3-example"},{"type":"function","title":"Jido.Error.workflow_error/3","doc":"Creates a new workflow error.\n\nUse this for general workflow-related errors that don't fit into other categories.","ref":"Jido.Error.html#workflow_error/3"},{"type":"function","title":"Parameters - Jido.Error.workflow_error/3","doc":"- `message`: A string describing the workflow error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#workflow_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.workflow_error/3","doc":"iex> Jido.Error.workflow_error(\"Workflow 'ProcessOrder' failed\", %{order_id: 12345})\n    %Jido.Error{\n      type: :workflow_error,\n      message: \"Workflow 'ProcessOrder' failed\",\n      details: %{order_id: 12345},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#workflow_error/3-example"},{"type":"type","title":"Jido.Error.error_type/0","doc":"Defines the possible error types in the Jido system.\n\n- `:invalid_action`: Used when a action is improperly defined or used.\n- `:invalid_sensor`: Used when a sensor is improperly defined or used.\n- `:bad_request`: Indicates an invalid request from the client.\n- `:validation_error`: Used when input validation fails.\n- `:config_error`: Indicates a configuration issue.\n- `:execution_error`: Used when an error occurs during workflow execution.\n- `:workflow_error`: General workflow-related errors.\n- `:internal_server_error`: Indicates an unexpected internal error.\n- `:timeout`: Used when an workflow exceeds its time limit.\n- `:invalid_async_ref`: Indicates an invalid asynchronous workflow reference.\n- `:compensation_error`: Indicates an error occurred during compensation.","ref":"Jido.Error.html#t:error_type/0"},{"type":"type","title":"Jido.Error.t/0","doc":"Represents a structured error in the Jido system.\n\nFields:\n- `type`: The category of the error (see `t:error_type/0`).\n- `message`: A human-readable description of the error.\n- `details`: Optional map containing additional error context.\n- `stacktrace`: Optional list representing the error's stacktrace.","ref":"Jido.Error.html#t:t/0"},{"type":"module","title":"Jido.Util","doc":"A collection of utility functions for the Jido framework.\n\nThis module provides various helper functions that are used throughout the Jido framework,\nincluding:\n\n- ID generation\n- Name validation\n- Error handling\n- Logging utilities\n\nThese utilities are designed to support common operations and maintain consistency\nacross the Jido ecosystem. They encapsulate frequently used patterns and provide\na centralized location for shared functionality.\n\nMany of the functions in this module are used internally by other Jido modules,\nbut they can also be useful for developers building applications with Jido.","ref":"Jido.Util.html"},{"type":"macro","title":"Jido.Util.debug/2","doc":"","ref":"Jido.Util.html#debug/2"},{"type":"macro","title":"Jido.Util.error/2","doc":"","ref":"Jido.Util.html#error/2"},{"type":"function","title":"Jido.Util.generate_id/0","doc":"Generates a unique ID.","ref":"Jido.Util.html#generate_id/0"},{"type":"function","title":"Jido.Util.log/4","doc":"","ref":"Jido.Util.html#log/4"},{"type":"function","title":"Jido.Util.validate_name/1","doc":"Validates the name of a Action.\n\nThe name must contain only letters, numbers, and underscores.","ref":"Jido.Util.html#validate_name/1"},{"type":"function","title":"Parameters - Jido.Util.validate_name/1","doc":"- `name`: The name to validate.","ref":"Jido.Util.html#validate_name/1-parameters"},{"type":"function","title":"Returns - Jido.Util.validate_name/1","doc":"- `{:ok, name}` if the name is valid.\n- `{:error, reason}` if the name is invalid.","ref":"Jido.Util.html#validate_name/1-returns"},{"type":"function","title":"Examples - Jido.Util.validate_name/1","doc":"iex> Jido.Action.validate_name(\"valid_name_123\")\n    {:ok, \"valid_name_123\"}\n\n    iex> Jido.Action.validate_name(\"invalid-name\")\n    {:error, %Jido.Error{type: :validation_error, message: \"The name must contain only letters, numbers, and underscores.\"}}","ref":"Jido.Util.html#validate_name/1-examples"},{"type":"function","title":"Jido.Util.validate_planner/1","doc":"","ref":"Jido.Util.html#validate_planner/1"},{"type":"extras","title":"README","doc":"Jido (自動) \n \n\n \n  自動 (Jido) - A flexible framework for building distributed Agents and Workflows in Elixir.","ref":"readme.html"},{"type":"extras","title":"Features - README","doc":"- **Actions**: Discrete, composable units of functionality with consistent interfaces\n- **Workflows**: Robust execution runtime with logging, telemetry, and error handling\n- **Agents**: Stateful autonomous entities that can plan and execute workflows\n- **Sensors**: Event-driven data gathering components\n- **Signals**: Cloud Events-based messaging between components\n- **Flexible Planning**: Pluggable planners for agent decision making\n- **Comprehensive Testing**: Rich testing tools and helpers\n- **Observable**: Built-in telemetry and debugging tools","ref":"readme.html#features"},{"type":"extras","title":"Installation - README","doc":"Add `jido` to your dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:jido, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Getting Started - README","doc":"","ref":"readme.html#getting-started"},{"type":"extras","title":"Creating an Action - README","doc":"Actions are the basic building blocks in Jido. Here's a simple arithmetic action:\n\n```elixir\ndefmodule MyApp.Actions.Add do\n  use Jido.Action,\n    name: \"add\",\n    description: \"Adds two numbers\",\n    schema: [\n      value: [type: :number, required: true],\n      amount: [type: :number, required: true]\n    ]\n\n  @impl true \n  def run(%{value: value, amount: amount}, _context) do\n    {:ok, %{result: value + amount}}\n  end\nend\n```","ref":"readme.html#creating-an-action"},{"type":"extras","title":"Creating a Simple Agent - README","doc":"Agents combine actions into autonomous behaviors:\n\n```elixir\ndefmodule MyApp.SimpleAgent do\n  use Jido.Agent,\n    name: \"SimpleBot\",\n    description: \"A simple agent that performs basic tasks\",\n    schema: [\n      location: [type: :atom, default: :home],\n      battery_level: [type: :integer, default: 100]\n    ]\n\n  @impl true\n  def plan(%__MODULE__{} = agent) do\n    {:ok,\n     %Jido.ActionSet{\n       agent: agent,\n       plan: [\n         {MyApp.Actions.Basic.Log, message: \"Hello, world!\"},\n         {MyApp.Actions.Basic.Sleep, duration: 50},\n         {MyApp.Actions.Basic.Log, message: \"Goodbye, world!\"}\n       ]\n     }}\n  end\nend\n```","ref":"readme.html#creating-a-simple-agent"},{"type":"extras","title":"Starting an Agent Worker - README","doc":"Start an agent worker under your supervision tree:\n\n```elixir\n# In your application.ex\nchildren = [\n  {Registry, keys: :unique, name: Jido.AgentRegistry},\n  {Jido.Agent.Supervisor, pubsub: MyApp.PubSub}\n]\n\n# Start an agent instance\n{:ok, pid} = Jido.Agent.Worker.start_link(MyApp.SimpleAgent.new())\n```","ref":"readme.html#starting-an-agent-worker"},{"type":"extras","title":"Contributing - README","doc":"We welcome contributions! Please feel free to submit a PR.\n\nTo run tests:\n\n```bash\nmix test\n```","ref":"readme.html#contributing"},{"type":"extras","title":"License - README","doc":"Apache License 2.0 - See [LICENSE.md](LICENSE.md) for details.","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}